////////////////////////////////////////////////////////////////////////////////
// File:	DirectoryParts.cs
//
// Created: 24th September 2003
// Updated: 15th March 2005
//
////////////////////////////////////////////////////////////////////////////////

namespace recls
{
	using System;
	using System.Collections;

	/// <summary>
	/// Provides access to the directory parts
	/// </summary>
	/// <remarks>This class holds the individual components of the directory for a particular FileEntry</remarks>
	public class DirectoryParts
		: IEnumerable
	{
		internal DirectoryParts(IntPtr entry)
		{
			m_entry		=	recls_api.CopyEntry(entry);
			m_cParts	=	recls_api.GetEntryDirectoryPartsCount(m_entry);
		}
		/// <summary>Releases the resources of the instance</summary>
		/// <remarks>The finaliser causes the release of the underlying <b>recls</b> library's resources</remarks>
		~DirectoryParts()
		{
			Dispose();
		}
		/// <summary>
		/// Releases the recls library's resources used by this instance
		/// </summary>
		/// <value>This method does not return a value</value>
		/// <remarks>Calling <b>Dispose</b> allows for an early release of resources back to the <b>recls</b> library</remarks>
		public void Dispose()
		{
			recls_api.CloseEntry(ref m_entry);
		}

		/// <summary>
		/// The number of parts
		/// </summary>
		/// <value>The number of parts in the directory</value>
		/// <remarks>If the search was not conducted with the <see cref="RECLS_FLAG.RECLS_F_DIRECTORY_PARTS"/> flag, then
		/// this property will always be 0</remarks>
		public int Count
		{
			get
			{
				return m_cParts;
			}
		}	

		/// <summary>
		/// Accesses a directory part.
		/// </summary>
		/// <value>Accesses a directory part</value>
		/// <param name="index">The index of the directory part to retrieve</param>
		/// <remarks>Throws an <c>IndexOutOfRangeException</c> if index is not within the valid range, as indicated by <see cref="Count"/>.</remarks>
		public string this[int index]
		{
			get
			{
				if(	index < 0 ||
					!(index < m_cParts))
				{
					throw new IndexOutOfRangeException();
				}

				return recls_api.GetEntryDirectoryPart(m_entry, index);
			}
		}

		/// <summary>
		/// Creates an enumerator for the entries matching the given <c>DirectoryParts</c>
		/// </summary>
		/// <returns>An IEnumerator for the DirectoryParts.</returns>
		/// <remarks>This method enables a DirectoryParts instance to be used in a 
		/// foreach expression, as in:
		/// <code>
		/// 
		/// DirectoryParts parts = . . .;
		/// foreach(string part in parts)
		/// {
		///   Console.WriteLine(part);
		/// }
		/// </code>
		/// </remarks>
		public IEnumerator GetEnumerator()
		{
			return new Enumerator(m_entry);
		}

		private sealed class Enumerator
			: IEnumerator
			, IDisposable
		{
			internal Enumerator(IntPtr entry)
			{
				m_entry		=	recls_api.CopyEntry(entry);
				m_current	=	-1;
				m_cParts	=	recls_api.GetEntryDirectoryPartsCount(entry);
			}
			/// <summary>Releases the resources of the instance</summary>
			/// <remarks>The finaliser causes the release of the underlying <b>recls</b> library's resources</remarks>
			~Enumerator()
			{
				Dispose();
			}

			/// <summary>
			/// Releases the recls library's resources used by this instance
			/// </summary>
			/// <value>This method does not return a value</value>
			/// <remarks>Calling <b>Dispose</b> allows for an early release of resources back to the <b>recls</b> library</remarks>
			public void Dispose()
			{
				recls_api.CloseEntry(ref m_entry);
			}

			/// <summary>
			/// Gets the current element in the collection.
			/// </summary>
			public object Current
			{
				get
				{
					if(	m_current < 0 ||
						!(m_current < m_cParts))
					{
						throw new InvalidOperationException("Enumeration is positioned before the first element, or after the last element");
					}

					return recls_api.GetEntryDirectoryPart(m_entry, m_current);
				}
			}	

			/// <summary>
			/// Advances the enumerator to the next element of the collection.
			/// </summary>
			public bool MoveNext()
			{
				if(m_current == m_cParts)
				{
					return false;
				}
				else if(++m_current == m_cParts)
				{
					return false;
				}
				else
				{
					return true;
				}
			}

			/// <summary>
			/// Sets the enumerator to its initial position, which is before the first element in the collection.
			/// </summary>
			public void Reset()
			{
				m_current = -1;
			}

			private IntPtr	m_entry;
			private int		m_cParts;
			private int		m_current;
		}

		/// <summary>
		/// The entry
		/// </summary>
		private IntPtr	m_entry;

		private int		m_cParts;
	}
}
