////////////////////////////////////////////////////////////////////////////////
// File:	FtpSearch.cs
//
// Created: 3rd June 2004
// Updated: 15th March 2005
//
////////////////////////////////////////////////////////////////////////////////

namespace recls
{
	using System;
	using System.Collections;

	/// <summary>
	/// Represents a file search operation
	/// </summary>
	/// <remarks>Each FtpSearch instance captures the search directory, pattern and flags of a particular 
	/// search. The FtpSearch instance may then be used in two ways:
	/// <list type="number">
	/// <item><description>as the expression of a <c>foreach</c> statement</description></item>
	/// <item><description>as the generator of an <see cref="IEnumerator"/>, via the <see cref="GetEnumerator"/> method, which 
	/// may then be used explicitly to provide enumeration of the search results via the <c>MoveNext</c> and <c>Reset</c>
	/// methods, and the <c>Current</c> property.
	/// </description></item>
	/// </list>
	/// </remarks>
	public class FtpSearch
		: Search
	{
// There's an interesting design decision here. When creating a FtpSearch
// instance it is desirable to find out at that time whether the search
// parameters are going to lead to a valid search or not. Hence we wish to
// start the search in the constructor. However, there are two reasons why
// this cannot be the case:
//
// (i) An enuemarable type can be placed in multiple foreach statements. Would
// we create the initial search in the FtpSearch ctor, and subsequent
// enumerations in the enumerator object? This is unbalanced.
//
// (ii) The enumerator object should implement Reset(), which means that the
// enumerator object, rather than the enumerable type, should initialise a
// search.
//
// The consequence of leaving the search until used is that the exception
// will be thrown from within the foreach, rather than from the object's
// construction, which is certainly unappealing from a common-sense pov.


		/// <summary>
		/// Creates a search for the items identified by pattern in the directory rootDir, according to the given flags.
		/// </summary>
		/// <param name="host">host</param> The host on which to conduct the FTP search. May not be null or empty
		/// <param name="username">username</param> The username with which to logon. May be null or empty for unsecured access.
		/// <param name="password">password</param> The password with which to logon. May be null or empty for unsecured access.
		/// <param name="searchRoot">The root directory of the search, e.g. "h:\recls". If it is null, or the empty string, the current directory of the calling process is used</param>
		/// <param name="pattern">The search pattern, e.g. "*.*. If it is null, or the empty string, all files are searched.</param>
		/// <param name="flags">Members of the <see cref="RECLS_FLAG"/> enumeration, to alter the search. For example, 
		/// <see cref="RECLS_FLAG.RECLS_F_DIRECTORIES"/> causes directories to be retrieved; <see cref="RECLS_FLAG.RECLS_F_RECURSIVE"/> causes the 
		/// search to recurse all subdirectories of <c>searchRoot</c></param>
		/// <remarks>No search is conducted at the time of construction of a <see cref="FtpSearch"/> instance. Hence, no search-specific
		/// errors will be reported at this time. Only when the enumeration is conducted on the <see cref="IEnumerator"/> returned by
		/// the <see cref="GetEnumerator"/> method will such errors occur.</remarks>
		public FtpSearch(string host, string username, string password, string searchRoot, string pattern, RECLS_FLAG flags)
		{
			m_host 			= 	host;
			m_username 		= 	username;
			m_password 		= 	password;
			m_searchRoot 	= 	searchRoot;
			m_pattern		=   pattern;
			m_flags			=	flags;
		}

		/// <summary>
		/// Processes the matching entries by passing each one to the given delegate function
		/// </summary>
		/// <param name="host">host</param> The host on which to conduct the FTP search. May not be null or empty
		/// <param name="username">username</param> The username with which to logon. May be null or empty for unsecured access.
		/// <param name="password">password</param> The password with which to logon. May be null or empty for unsecured access.
		/// <param name="searchRoot">The root directory of the search, e.g. "h:\recls". If it is null, or the empty string, the current directory of the calling process is used</param>
		/// <param name="pattern">The search pattern, e.g. "*.*. If it is null, or the empty string, all files are searched.</param>
		/// <param name="flags">Members of the <see cref="RECLS_FLAG"/> enumeration, to alter the search. For example, 
		/// <see cref="RECLS_FLAG.RECLS_F_DIRECTORIES"/> causes directories to be retrieved; <see cref="RECLS_FLAG.RECLS_F_RECURSIVE"/> causes the 
		/// search to recurse all subdirectories of <c>searchRoot</c></param>
		/// <param name="fn">The delegate, of type <see cref="Search.ProcessEntry"/>ProcessEntry, that will receive each matching item</param>
		/// <returns>Returns true if the set of matching entries was processed to completion, or false if it was cancelled</returns>
		/// <remarks>This static method conducts a search matching the given parameters, and calls the given delegate function for 
		/// each matching item. The search is cancellable by means of return false from the delegate</remarks>
		public static bool Process(string host, string username, string password, string searchRoot, string pattern, RECLS_FLAG flags, ProcessEntry fn)
		{
			int num;

			return Process(host, username, password, searchRoot, pattern, flags, fn, out num);
		}

		/// <summary>
		/// Processes the matching entries by passing each one to the given delegate function
		/// </summary>
		/// <param name="host">host</param> The host on which to conduct the FTP search. May not be null or empty
		/// <param name="username">username</param> The username with which to logon. May be null or empty for unsecured access.
		/// <param name="password">password</param> The password with which to logon. May be null or empty for unsecured access.
		/// <param name="searchRoot">The root directory of the search, e.g. "h:\recls". If it is null, or the empty string, the current directory of the calling process is used</param>
		/// <param name="pattern">The search pattern, e.g. "*.*. If it is null, or the empty string, all files are searched.</param>
		/// <param name="flags">Members of the <see cref="RECLS_FLAG"/> enumeration, to alter the search. For example, 
		/// <see cref="RECLS_FLAG.RECLS_F_DIRECTORIES"/> causes directories to be retrieved; <see cref="RECLS_FLAG.RECLS_F_RECURSIVE"/> causes the 
		/// search to recurse all subdirectories of <c>searchRoot</c></param>
		/// <param name="fn">The delegate, of type <see cref="Search.ProcessEntry"/>ProcessEntry, that will receive each matching item</param>
		/// <param name="num">The number of matching elements processed</param>
		/// <returns>Returns true if the set of matching entries was processed to completion, or false if it was cancelled</returns>
		/// <remarks>This static method conducts a search matching the given parameters, and calls the given delegate function for 
		/// each matching item. The search is cancellable by means of return false from the delegate. The number of items reported is only
		/// those for which the delegate returned true; in other words, unvisited and cancelled entries are not counted.</remarks>
		public static bool Process(string host, string username, string password, string searchRoot, string pattern, RECLS_FLAG flags, ProcessEntry fn, out int num)
		{
			FtpSearch	fs	=	new FtpSearch(host, username, password, searchRoot, pattern, flags);

			num = 0;
			foreach(FileEntry entry in fs)
			{
				if(!fn(entry))
				{
					return false;
				}
				else
				{
					++num;
				}
			}

			return true;
		}


		/// <summary>
		/// Returns the enumerator with which the given search can be enumerated
		/// </summary>
		/// <returns>An IEnumerator for the entire FtpSearch.</returns>
		/// <remarks>This method enables a FtpSearch instance to be used in a 
		/// foreach expression, as in:
		/// <code>
		/// 
		/// FtpSearch search = . . .;
		/// foreach(<see cref="FileEntry"/> entry in search)
		/// {
		///   Console.WriteLine(entry.<see cref="FileEntry.Path"/>);
		/// }
		/// 
		/// </code>
		/// As with the <see cref="FtpSearch"/> constructor, no enumeration is carried out at this time, only
		/// when the returned enumerator is used.
		/// 
		/// When the enumerator's <c>MoveNext</c> method is called, it may throw a <see cref="ReclsException"/> if the
		/// search fails, except for the case where the search runs out of data. In this case, the enumerator's 
		/// <c>MoveNext</c> method returns false.
		/// </remarks>
		public override IEnumerator GetEnumerator()
		{
			return new Enumerator(m_host, m_username, m_password, m_searchRoot, m_pattern, m_flags);
		}

		private sealed class Enumerator
			: IEnumerator
			, IDisposable
		{
			/// Enumerator
			internal Enumerator(string host, string username, string password, string searchRoot, string pattern, RECLS_FLAG flags)
			{
				m_host 			= 	host;
				m_username 		= 	username;
				m_password 		= 	password;
				m_searchRoot 	= 	searchRoot;
				m_pattern		=   pattern;
				m_flags			=	flags;
				m_hSrch			=	IntPtr.Zero;
				m_entry			=	IntPtr.Zero;
				m_bComplete		=	false;
			}

			/// <summary>Releases the resources of the instance</summary>
			/// <remarks>The finaliser causes the release of the underlying <b>recls</b> library's resources</remarks>
			~Enumerator()
			{
				Dispose();
			}
			/// <summary>
			/// Releases the recls library's resources used by this instance
			/// </summary>
			/// <value>This method does not return a value</value>
			/// <remarks>Calling <b>Dispose</b> allows for an early release of resources back to the <b>recls</b> library</remarks>
			public void Dispose()
			{
				recls_api.CloseSearch(ref m_hSrch);
				recls_api.CloseEntry(ref m_entry);
			}

			/// <summary>
			/// Gets the current element in the collection.
			/// </summary>
			public object Current
			{
				get
				{
					if(IntPtr.Zero == m_entry)
					{
						throw new InvalidOperationException("Enumeration is positioned before the first element, or after the last element");
					}

					return new FileEntry(m_entry);
				}
			}

			/// <summary>
			/// Advances the enumerator to the next element of the collection.
			/// </summary>
			public bool MoveNext()
			{
				// MoveNext() must be called before Current is used, whether
				// that is for a newly created enumerator or after Reset() is
				// called. Hence,

				if(m_bComplete)
				{
					// Was completed on the last call.

					return false;
				}
				else
				{
					// Close the current entry
					recls_api.CloseEntry(ref m_entry);

					if(IntPtr.Zero == m_hSrch)
					{
						// Start a new enumeration
						m_hSrch = recls_api.OpenFtpSearch(m_host, m_username, m_password, m_searchRoot, m_pattern, m_flags);

						if(IntPtr.Zero == m_hSrch)
						{
							m_bComplete = true;

							return false;
						}
						else
						{
							// Call GetDetails
							recls_api.GetEntry(m_hSrch, out m_entry);
						}
					}
					else
					{
						// Enumeration is in progress, so just try and advance

						// Call GetNextDetails
						if(!recls_api.NextEntry(m_hSrch, out m_entry))
						{
							m_bComplete = true;

							return false;
						}
					}
				}

				return true;
			}

			/// <summary>
			/// Sets the enumerator to its initial position, which is before the first element in the collection.
			/// </summary>
			public void Reset()
			{
				// Since Reset() resets an enumerator to *exactly* the same
				// state as when newly created, we can just close anything
				// that's open.

				recls_api.CloseSearch(ref m_hSrch);
				recls_api.CloseEntry(ref m_entry);

				m_bComplete		=	false;
			}

			/// The search handle
			private IntPtr		m_hSrch;
			private IntPtr		m_entry;
			private string		m_host;
			private string		m_username;
			private string		m_password;
			private string		m_searchRoot;
			private string		m_pattern;
			private RECLS_FLAG	m_flags;
			private bool		m_bComplete;
		}

		private string		m_host;
		private string		m_username;
		private string		m_password;
		private string		m_searchRoot;
		private string		m_pattern;
		private RECLS_FLAG	m_flags;
	}
}
