/* /////////////////////////////////////////////////////////////////////////////
 * File:        Entry.java
 *
 * Purpose:     Implementation of the org.recls.Entry class
 *
 * Created      21st November 2003
 * Updated:     15th March 2005
 *
 * Author:      Matthew Wilson
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright 2002-2005, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.recls.org/
 *                          http://www.synesis.com.au/software
 *
 *              email:      admin@recls.org
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


package org.recls;

import java.util.Date;


/** This class represents an entry in the search
 */
public class Entry
{
    private Entry(int hEntry, String path, String[] directoryParts)
    {
        m_hEntry            =   hEntry;
        m_path              =   path;
        m_directoryParts    =   directoryParts;
    }
    /** Releases the resources owned by the entry
     *
     * \note It is perfectly legal to call this method more than once.
     */
    public native void close();
    /** Releases the resources owned by the entry
     */
    public void finalize()
    {
        close();
    }

// Properties
    /** The full path of the entry
     *
     * \note For "/usr/include/recls/recls_assert.h" this would yield "/usr/include/recls/recls_assert.h"
     */
    public String           getPath()
    {
        return m_path;
    }
    /** The drive component of the entry's path
     *
     * \note For "H:\Dev\include\recls\recls_assert.h" this would yield 'H'
     * \note Throws java.lang.UnsupportedOperationException if called on non-Win32 systems
     */
    public native char      getDrive(); // Throws an exception on 
    /** The directory component of the entry's path
     *
     * \note For "/usr/include/recls/recls_assert.h" this would yield "/usr/include/recls/"
     */
    public native String    getDirectory();
    /** The full location component of the entry's path.
     *
     * \note This is everything before the filename+fileext. On Win32 systems for "H:\Dev\include\recls\recls_assert.h" this would yield "H:\Dev\include\recls\"
     * \note On systems that do not have multiple drives, this returns the same value as getDirectory()
     */
    public native String    getDirectoryPath();
    /** The UNC part, if any, of the entry's path.
     *
     * \note This is only supported on Win32 systems. On other systems it will return an empty string
     */
    public native String    getUNCDrive();
    /** An array of Strings representing the parts of the Directory property
     *
     * \note For "/usr/include/recls/recls_assert.h" this would yield [ "/", "usr/", "include/", "recls/"]
     */
    public String[]         getDirectoryParts()
    {
        return m_directoryParts;
    }
    /** The file component of the entry's path
     *
     * \note For "/usr/include/recls/recls_assert.h" this would yield "recls_assert.h"
     */
    public native String    getFile();
    /** The short equivalent of the entry's File property
     *
     * \note On Win32 systems, this is the 8.3 form, e.g. "recls_~1.h". On other systems this is identical to the File property
     */
    public native String    getShortFile();
    /** The file name component of the entry's path
     *
     * \note For "/usr/include/recls/recls_assert.h" this would yield "recls_assert"
     */
    public native String    getFileName();
    /** The file extension component of the entry's path
     *
     * \note For "/usr/include/recls/recls_assert.h" this would yield "h"
     */
    public native String    getFileExt();
    /** The time the entry was created
     */
    public Date             getCreationTime()
    {
        return new Date(getCreationTime_());
    }
    /** The time the entry was last modified
     */
    public Date             getModificationTime()
    {
        return new Date(getModificationTime_());
    }
    /** The time the entry was last accessed
     */
    public Date             getLastAccessTime()
    {
        return new Date(getLastAccessTime_());
    }
    /** The time the entry's last status changed
     */
    public Date             getLastStatusChangeTime()
    {
        return new Date(getLastStatusChangeTime_());
    }
    /** The size of the entry
     */
    public native long      getSize();
    /** Indicates whether the entry is read-only
     */
    public native boolean   isReadOnly();
    /** Indicates whether the entry is a directory
     */
    public native boolean   isDirectory();
    /** Indicates whether the entry is a link
     */
    public native boolean   isLink();
    /** Indicates whether the entry is a link
     */
    public native boolean   isUNC();

    /** Provides a String representation of the entry
     */
    public String toString()
    {
        return m_path;
    }

// Implementation methods
    public native long      getCreationTime_();
    public native long      getModificationTime_();
    public native long      getLastAccessTime_();
    public native long      getLastStatusChangeTime_();

// Members
    private int         m_hEntry;   // The opaque handle to the entry
    private String      m_path;
    private String[]    m_directoryParts;
}

