################################################################################
# File:     test.rb
#
# Purpose:  Test program for the recls-Ruby mapping
#
# Created:  15th March 2004
# Updated:  12th April 2004
#
# Author:   Matthew Wilson
#
# License:  (Licensed under the Synesis Software Open License)
#
#           Copyright (C) 1999-$$PROJ_YEAR$$, Synesis Software Pty Ltd.
#           All rights reserved.
#
#           www:        http://recls.org
#                       http://synesis.com.au/software
#
#           email:      software@synesis.com.au
#
################################################################################

################################################################################
# Imports

require "recls"

################################################################################
# Functions

# Displays the usage, and optionally aborts the program

def usage(do_abort)
    print "recls Ruby Test Program\n\n"
    print "Usage: test.rb [-d] [-f] [-h] [-p] [-s] [-R] [-H<host>] <pattern> [<root-dir>]\n"
    printf "\t-d               -   search for directories\n"
    printf "\t-f               -   search for files (this is the default, if -d is not specified)\n"
    printf "\t-h               -   search in the roots of all drives on the system. Ignore <root-dir>\n"
    printf "\t-H               -   search the FTP <host>\n"
    printf "\t-p               -   evaluate and display directory parts\n"
    printf "\t-s               -   succinct; shows only the path\n"
    printf "\t-R               -   does not recurse; recursive search is the default\n"
    printf "\t<pattern>        -   search pattern, e.g. \"*.cpp\" (default is to search for all files)\n"
    printf "\t<root-dir>       -   root directory of search; default is current working directory\n"
    if(do_abort)
        abort
    end
end # usage()


# Performs the search, within the given root, according to the given criteria

def doSearch(host, username, password, root, pattern, flags, succinct)

	if host
    	fs = Recls::FtpSearch::new(host, username, password, root, pattern, flags)
	else
    	fs = Recls::FileSearch::new(root, pattern, flags)
	end

    print "Searching for \"" + pattern + "\" in \"" + root + "\"\n" if !succinct

    # Local function, used to pad the output

    def getPad(len)
        return "".rjust(len)
    end

    # The enumeration of the contents, and their display

    prefix = succinct ? "" : "  ";

    fs.each \
    { |fe| 
        puts prefix + String(fe)
        if !succinct
            puts prefix + fe.directoryPath

            puts prefix + fe.drive

            puts prefix + "  " + fe.directory

            # Directory parts
            dir_off = 2
            fe.directoryParts.each \
            { |dp| 
                puts prefix + getPad(dir_off) + dp
                dir_off = dir_off + dp.length
            }

            # file, shortFile, fileName, fileExt
            file_off = 2 + fe.directory.length
            puts prefix + getPad(file_off) + fe.file
            if fe.shortFile.length > 0
                puts prefix + getPad(file_off) + fe.shortFile
            end
            puts prefix + getPad(file_off) + fe.fileName
            if fe.fileExt.length > 0
                puts prefix + getPad(file_off + fe.fileName.length) + "." + fe.fileExt
            end

            puts prefix + "  " + String(fe.size) + " byte(s)"

            puts prefix + "  " + "Created:            " + fe.creationTime.strftime("%x %X")
            puts prefix + "  " + "Modified:           " + fe.modificationTime.strftime("%x %X")
            puts prefix + "  " + "Last Accessed:      " + fe.lastAccessTime.strftime("%x %X")
            puts prefix + "  " + "Last Status Change: " + fe.lastStatusChangeTime.strftime("%x %X")

            puts prefix + "  " + "Read-only"   if fe.isReadOnly
            puts prefix + "  " + "Directory"   if fe.isDirectory
            puts prefix + "  " + "Link"        if fe.isLink

        end
    }

    # Collecting the names only
#   names = fs.collect { |fe| fe.file }
#   puts "Collected files: "
#   names.each { |name| puts "  " + name }

    # Shows the 'min' and 'max' values
#   puts "'Minimum' item:"
#   puts "  " + fs.min.to_s
#   puts "'Maximum' item:"
#   puts "  " + fs.max.to_s

    # Selecting the read-only items
#   puts "Read-only items:"
#   ro_items = fs.select { |fe| fe.isReadOnly }
#   ro_items.each { |fe| puts "  " + fe.to_s }

    # 'Sorting' the items
#   puts "Sorted items:"
#   sorted_items = fs.sort
#   sorted_items.each { |fe| puts "  " + fe.to_s }

end # doSearch()

################################################################################
# main

flags           =   Recls::RECLS_F_RECURSIVE    # Default is recursive
succinct        =   false                       # Default is verbose
bAllHardDrives  =   false                       # Default is to use a single root

host			=	nil
username		=	""
password		=	""

# Select the options
options     =   ARGV.select do |o|
    o[0, 1] == '-'
end

# Select the arguments
args        =   ARGV.select do |o|
    o[0, 1] != '-'
end

# Process the options
options.each \
{ |arg| 
    succinct = true if arg == "-s"
    flags |= Recls::RECLS_F_FILES           if arg == "-f"
    flags |= Recls::RECLS_F_DIRECTORIES     if arg == "-d"
    flags |= Recls::RECLS_F_DIRECTORY_PARTS if arg == "-p"
    flags &= ~Recls::RECLS_F_RECURSIVE      if arg == "-D"
    bAllHardDrives = true                   if arg == "-h"
	host = arg.slice(2..arg.length)			if arg.slice(0 .. 1) == "-H"
    if arg == "-?"
        usage(true)
    end
}

usage(true) if 0 == args.length

pattern     =   args.shift
searchRoot  =   args.shift || ""

# Now either process for roots, or the given root

print "recls version: " + Recls::RECLS_VERSION + "\n" if !succinct

if host
		#		searchRoot = "/"
end

if(bAllHardDrives)
    Recls.roots.each \
    { | root |
        doSearch(host, username, password, root, pattern, flags, succinct)
    }
else
    doSearch(host, username, password, searchRoot, pattern, flags, succinct)
end

################################################################################
