/* /////////////////////////////////////////////////////////////////////////////
 * File:        ReclsFileSearchDirectoryNode.h
 *
 * Purpose:     ReclsFileSearchDirectoryNode class.
 *
 * Created:     31st May 2004
 * Updated:     19th May 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2004-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/** \file ReclsFileSearchDirectoryNode.h Definition of the ReclsFileSearchDirectoryNode class
 */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

// recls includes
#include <recls.h>
#include "recls_string.h"
#include "recls_util.h"

// STLSoft includes
#ifdef _RECLS_SUPPORTS_MULTIPATTERN
# include <stlsoft/searchspec_sequence.hpp>
#endif /* _RECLS_SUPPORTS_MULTIPATTERN */

// Platform-specific includes
#if defined(RECLS_PLATFORM_IS_UNIX)

# include <unixstl.h>
# include <unixstl/filesystem_traits.hpp>
# include <unixstl/file_path_buffer.hpp>
# include <unixstl/glob_sequence.hpp>

namespace platform_stl = ::unixstl;

#elif defined(RECLS_PLATFORM_IS_WIN32)

# include <winstl.h>
# include <winstl/filesystem_traits.hpp>
# include <winstl/file_path_buffer.hpp>
# include <winstl/findfile_sequence.hpp>

namespace platform_stl = ::winstl;

#else /* unrecognised platform */

# error The platform is not recognised. Only RECLS_PLATFORM_IS_UNIX or RECLS_PLATFORM_IS_WIN32 currently supported

#endif /* platform */

#include "ReclsSearch.hpp"

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{
#endif /* !RECLS_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

class ReclsFileSearchDirectoryNode
    : public ReclsSearchDirectoryNode
{
public:
    typedef ReclsFileSearchDirectoryNode                                        class_type;
    typedef ::platform_stl::filesystem_traits<recls_char_t>                     traits_type;
    typedef ::platform_stl::basic_file_path_buffer<recls_char_t>                file_path_buffer_type;
private:
    typedef RECLS_STRING_TEMPLATE_1(recls_char_t)                               string_type;
#if defined(RECLS_PLATFORM_IS_UNIX)
    typedef unixstl_ns_qual(glob_sequence)                                      file_find_sequence_type;
#elif defined(RECLS_PLATFORM_IS_WIN32)
    typedef winstl_ns_qual(basic_findfile_sequence)<recls_char_t, traits_type>  file_find_sequence_type;
#else /* unrecognised platform */
# error The platform is not recognised. Only RECLS_PLATFORM_IS_UNIX or RECLS_PLATFORM_IS_WIN32 currently supported
#endif /* platform */
    typedef file_find_sequence_type                                             directory_sequence_type;
#ifdef _RECLS_SUPPORTS_MULTIPATTERN
    typedef stlsoft_ns_qual(searchspec_sequence)<file_find_sequence_type>       entry_sequence_type;
#else /* ? _RECLS_SUPPORTS_MULTIPATTERN */
    typedef file_find_sequence_type                                             entry_sequence_type;
#endif /* _RECLS_SUPPORTS_MULTIPATTERN */

// Construction
protected: // Not private, or GCC whines
    ReclsFileSearchDirectoryNode(recls_uint32_t flags, recls_char_t const *rootDir, recls_char_t const *pattern);
public:
    virtual ~ReclsFileSearchDirectoryNode();

    static class_type *FindAndCreate(recls_uint32_t flags, recls_char_t const *rootDir, recls_char_t const *pattern);
    static class_type *FindAndCreate(recls_uint32_t flags, recls_char_t const *rootDir, recls_char_t const *subDir, recls_char_t const *pattern);

    static recls_rc_t Stat( recls_char_t const  *path
                        ,   recls_uint32_t      flags
                        ,   recls_info_t        *phEntry);

// ReclsSearchDirectoryNode methods
private:
    /* virtual */ recls_rc_t GetNext();
    /* virtual */ recls_rc_t GetDetails(recls_info_t *pinfo);
    /* virtual */ recls_rc_t GetNextDetails(recls_info_t *pinfo);

// Implementation
private:
    recls_rc_t      Initialise();

#ifdef RECLS_ENFORCING_CONTRACTS
    recls_bool_t    is_valid() const;
#endif /* RECLS_ENFORCING_CONTRACTS */

#if defined(RECLS_COMPILER_IS_BORLAND)
    static directory_sequence_type::const_iterator _select_iter(int b, directory_sequence_type::const_iterator trueVal, directory_sequence_type::const_iterator falseVal)
    {
        // I can't explain it, but Borland does not like the tertiary operator and the copy-ctors of the iterators
        if(b)
        {
            return trueVal;
        }
        else
        {
            return falseVal;
        }
    }
#endif /* !RECLS_COMPILER_IS_BORLAND */
    static int _ssFlags_from_reclsFlags(recls_uint32_t flags)
    {
        int ssFlags = 0;

        if(0 != (flags & RECLS_F_FILES))
        {
#ifdef _RECLS_SUPPORTS_MULTIPATTERN
            // Because Digital Mars 8.40- has a problem, we must access the typedef separately from the enum value
            typedef entry_sequence_type::find_sequence_type sequence_t;

            ssFlags |= sequence_t::files;

//            ssFlags |= entry_sequence_type::find_sequence_type::files;
#else /* ? _RECLS_SUPPORTS_MULTIPATTERN */
            ssFlags |= entry_sequence_type::files;
#endif /* _RECLS_SUPPORTS_MULTIPATTERN */
        }
        if(0 != (flags & RECLS_F_DIRECTORIES))
        {
#ifdef _RECLS_SUPPORTS_MULTIPATTERN
            // Because Digital Mars 8.40- has a problem, we must access the typedef separately from the enum value
            typedef entry_sequence_type::find_sequence_type sequence_t;

            ssFlags |= sequence_t::directories;

//            ssFlags |= entry_sequence_type::find_sequence_type::directories;
#else /* ? _RECLS_SUPPORTS_MULTIPATTERN */
            ssFlags |= entry_sequence_type::directories;
#endif /* _RECLS_SUPPORTS_MULTIPATTERN */
        }

        return ssFlags;
    }

// Members
private:
    recls_info_t                            m_current;
    class_type                              *m_dnode;
    recls_uint32_t const                    m_flags;
    file_path_buffer_type                   m_rootDir;
    string_type                             m_pattern;
    directory_sequence_type                 m_directories;
    directory_sequence_type::const_iterator m_directoriesBegin;
    entry_sequence_type                     m_entries;
    entry_sequence_type::const_iterator     m_entriesBegin;

// Not to be implemented
private:
    ReclsFileSearchDirectoryNode(class_type const &);
    class_type &operator =(class_type const &);
};

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */
