/* /////////////////////////////////////////////////////////////////////////////
 * File:        ReclsFtpSearch_win32.cpp
 *
 * Purpose:     Implementation of the ReclsFtpSearch class for Win32.
 *
 * Created:     16th August 2003
 * Updated:     19th May 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright 2003-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include <recls.h>
#include <recls_assert.h>
#include "recls_util.h"
#include "recls_impl.h"

#include <exception>

#include <stlsoft/auto_buffer.hpp>
#include <stlsoft/exceptions.hpp>

#include <winstl/processheap_allocator.hpp>

#if defined(RECLS_DELAY_LOAD_WININET)
# ifdef INETSTL_INCL_H_INETSTL
#  error INETSTL_INCL_H_INETSTL
# endif /* INETSTL_INCL_H_INETSTL */
# include "recls_wininet_dl.h"
#endif /* RECLS_DELAY_LOAD_WININET */

#include <inetstl_connection.h>
#include <inetstl_filesystem_traits.h>

#ifdef RECLS_USE_NEW_INETSTL_FFS
# include <inetstl/findfile_sequence.hpp>
#else /* ? RECLS_USE_NEW_INETSTL_FFS */
# include <inetstl_findfile_sequence.h>
# ifdef _RECLS_SUPPORTS_MULTIPATTERN
#  include <inetstl_searchspec_sequence.h>
# endif /* _RECLS_SUPPORTS_MULTIPATTERN */
#endif /* RECLS_USE_NEW_INETSTL_FFS */

#include <inetstl_session.h>

#include "recls_debug.h"

#include "ReclsSearch.hpp"
#include "ReclsFtpSearch.hpp"
#include "ReclsFtpSearchDirectoryNode_win32.hpp"

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{
#endif /* !RECLS_NO_NAMESPACE */

namespace {

typedef ReclsFtpSearchDirectoryNode::traits_type                    traits_type;

typedef stlsoft_ns_qual(auto_buffer)<       recls_char_t
                                        ,   winstl_ns_qual(processheap_allocator)<recls_char_t>
                                        >                           buffer_t;

typedef inetstl_ns_qual(basic_session)<     recls_char_t
                                        ,   stlsoft_ns_qual(null_exception_policy)
                                        >                           session_type;
typedef inetstl_ns_qual(basic_connection)<  recls_char_t
                                        ,   stlsoft_ns_qual(null_exception_policy)
                                        >                           connection_type;

stlsoft_ns_using(c_str_ptr)

} // anonymous namespace

/* /////////////////////////////////////////////////////////////////////////////
 * Utility functions
 */

inline size_t count_dir_parts(recls_char_t const *begin, recls_char_t const *end)
{
    return count_char_instances(begin, end, traits_type::path_name_separator());
}

/* /////////////////////////////////////////////////////////////////////////////
 * ReclsFtpSearch
 */

inline void *ReclsFtpSearch::operator new(size_t cb, int cDirParts, size_t cbRootDir)
{
    function_scope_trace("ReclsFtpSearch::operator new");

    cbRootDir = align_up_size(cbRootDir);

    recls_assert(cb > stlsoft_raw_offsetof(ReclsFtpSearch, data));

    cb = stlsoft_raw_offsetof(ReclsFtpSearch, data)
       + (cDirParts) * sizeof(recls_strptrs_t)
       + cbRootDir;

    void    *pv =   malloc(cb);

#ifdef RECLS_COMPILER_THROWS_ON_NEW_FAIL
    if(NULL == pv)
    {
        throw std::bad_alloc();
    }
#endif /* RECLS_COMPILER_THROWS_ON_NEW_FAIL */

    return pv;
}

#ifdef RECLS_COMPILER_REQUIRES_MATCHING_PLACEMENT_DELETE
void ReclsFtpSearch::operator delete(void *pv, int /* cDirParts */, size_t /* cbRootDir */)
{
    function_scope_trace("ReclsFtpSearch::operator delete");

    free(pv);
}
#endif /* RECLS_COMPILER_REQUIRES_MATCHING_PLACEMENT_DELETE */

void ReclsFtpSearch::operator delete(void *pv)
{
    function_scope_trace("ReclsFtpSearch::operator delete");

    free(pv);
}

/* static */ recls_rc_t ReclsFtpSearch::FindAndCreate(  recls_char_t const  *host
                                                    ,   recls_char_t const  *username
                                                    ,   recls_char_t const  *password
                                                    ,   recls_char_t const  *rootDir
                                                    ,   recls_char_t const  *pattern
                                                    ,   recls_uint32_t      flags
                                                    ,   ReclsFtpSearch      **ppsi)
{
    function_scope_trace("ReclsFtpSearch::FindAndCreate");

    debug_printf("host:    %s\n", host);
    debug_printf("rootDir: %s\n", rootDir);
    debug_printf("pattern: %s\n", pattern);

    // The first thing to do is to open the session and connection

    recls_assert(NULL != host);
    recls_assert(NULL != rootDir);
    recls_assert(NULL != pattern);
    recls_assert(NULL != ppsi);

    *ppsi = NULL;

    recls_rc_t      rc;
    session_type    session(RECLS_LITERAL("recls-enumerator"), INTERNET_OPEN_TYPE_DIRECT, NULL, NULL, 0);

    // The session and connection are done in an exception-free manner just to 
    // avoid CRT linking issues. It's not the best way to write software in
    // general.

    if(!session)
    {
        debug_printf("Failed to create sessions: %lu\n", ::GetLastError());

        rc = RECLS_RC_FTP_INIT_FAILED;
    }
    else
    {
        DWORD   dwFlags =   0;

        if(flags & RECLS_F_PASSIVE_FTP)
        {
            dwFlags |= INTERNET_FLAG_PASSIVE;
        }

        connection_type connection(session, host, INTERNET_DEFAULT_FTP_PORT, username, password, INTERNET_SERVICE_FTP, dwFlags);

        if(!connection)
        {
            DWORD   dwErr = connection.last_error();

            debug_printf("Failed to open connection: %lu\n", dwErr);

            rc = RECLS_RC_FTP_CONNECTION_FAILED;
        }
        else
        {
            ReclsFtpSearch      *si;
            size_t              cchFullPath =   traits_type::get_full_path_name(connection, rootDir, 0, NULL);
            buffer_t            fullPath(1 + cchFullPath + 1); // +1 (nul) +1 (dir-end)

            if(0 == cchFullPath /* ||
                !traits_type::file_exists(fullPath) */)
            {
                debug_printf("Invalid directory\n");

                rc = RECLS_RC_INVALID_DIRECTORY;
            }
            else
            {
                traits_type::get_full_path_name(connection, rootDir, fullPath.size(), &fullPath[0]);

                traits_type::ensure_dir_end(&fullPath[0]);

                size_t  lenSearchRoot   =   traits_type::str_len(&fullPath[0]);

                recls_assert(0 < lenSearchRoot);

                rootDir = &fullPath[0];

                // Count the directory parts. This is always done for the ReclsFtpSearch class, since it
                // uses them to recurse.
                recls_char_t const          *begin      =   rootDir;
                recls_char_t const *const   end         =   rootDir + lenSearchRoot;
                int                         cDirParts   =   count_dir_parts(begin, end);

#ifdef RECLS_COMPILER_THROWS_ON_NEW_FAIL
                try
                {
#endif /* RECLS_COMPILER_THROWS_ON_NEW_FAIL */
                    si = new(cDirParts, sizeof(recls_char_t) * (1 + lenSearchRoot)) ReclsFtpSearch(session, connection, cDirParts, rootDir, pattern, flags);
#ifdef RECLS_COMPILER_THROWS_ON_NEW_FAIL
                }
                catch(std::bad_alloc &)
                {
                    debug_printf("Allocation failed\n");

                    si = NULL;
                }
#endif /* RECLS_COMPILER_THROWS_ON_NEW_FAIL */

                if(NULL == si)
                {
                    rc = RECLS_RC_FAIL;
                }
                else
                {
                    connection.detach();
                    session.detach();

                    // This is a nasty hack. It's tantamount to ctor & create function, so
                    // should be made more elegant soon.
                    if(NULL == si->m_dnode)
                    {
                        delete si;

//                      si = NULL;

                        debug_printf("No more data\n");

                        rc = RECLS_RC_NO_MORE_DATA;
                    }
                    else
                    {
                        *ppsi = si;

                        rc = RECLS_RC_OK;
                    }
                }
            }
        }
    }

    return rc;
}

recls_char_t const *ReclsFtpSearch::calc_rootDir_(int cDirParts, recls_char_t const *rootDir)
{
    function_scope_trace("ReclsFtpSearch::calc_rootDir_");

    // Root dir is located after file parts, and before pattern
    return traits_type::str_copy(::stlsoft::sap_cast<recls_char_t*>(&data[cDirParts * sizeof(recls_strptrs_t)]), rootDir);
}

ReclsFtpSearch::ReclsFtpSearch( HINTERNET           hSess
                            ,   HINTERNET           hConn
                            ,   int                 cDirParts
                            ,   recls_char_t const  *rootDir
                            ,   recls_char_t const  *pattern
                            ,   recls_uint32_t      flags)
    : m_session(hSess)
    , m_connection(hConn)
    , m_flags(flags)
    , m_rootDir(calc_rootDir_(cDirParts, rootDir))
{
    function_scope_trace("ReclsFtpSearch::ReclsFtpSearch");

    recls_assert(NULL != hSess);
    recls_assert(NULL != hConn);
    recls_assert(NULL != rootDir);
    recls_assert(NULL != pattern);

    // Initialise the directory parts.

    recls_message_assert("FTP filesystems do not allow drive specifications", rootDir[1] != ':');

    m_dnode = ReclsFtpSearchDirectoryNode::FindAndCreate(m_connection, m_flags, rootDir, pattern);
}

ReclsFtpSearch::~ReclsFtpSearch()
{
    if(NULL != m_connection)
    {
        ::InternetCloseHandle(m_connection);
    }
    if(NULL != m_session)
    {
        ::InternetCloseHandle(m_session);
    }
}

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */
